﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.Objects;
using System.Data;

using DomainModel;
using IdeaBlade.Core;
using IdeaBlade.EntityModel;

namespace Console01 {
  class Program {
    static void Main(string[] args) {
      new Program().DoIt();
    }

    private void DoIt() {
      _context = new DomainModelEntityManager();
      Customer aCustomer = _context.Customers.Where(c=> c.Orders.Any()).FirstOrNullEntity();

      if (aCustomer.Orders.Count > 0) {
        Console.WriteLine("Got customer " + aCustomer.CompanyName);
        Console.WriteLine("Customer has " + aCustomer.Orders.Count + " orders.");
        int sumOfOrderDetails = 0;
        int internationalOrders = 0;
        foreach (Order anOrder in aCustomer.Orders) {
          sumOfOrderDetails += anOrder.OrderDetails.Count();
          internationalOrders += (anOrder.InternationalOrder.EntityAspect.IsNullEntity ? 0 : 1);
        }
        Console.WriteLine("Customer has " + sumOfOrderDetails + " total line items (OrderDetails).");
        Console.WriteLine("Customer has " + internationalOrders + " total internatinal orders.");

        // Delete only the customer.
        aCustomer.EntityAspect.Delete();

        // Cascade rules will cascade the delete to objects in entity cache.
        // Use FindEntities to find all deleted objects.
        IEnumerable<Object> deletedObjects = _context.FindEntities(IdeaBlade.EntityModel.EntityState.Deleted).Cast<Object>();
        int numberOfObjectsDeleted = deletedObjects.Count();

        Console.WriteLine(numberOfObjectsDeleted + " entities were marked for deletion.");

        // We won't delete the graph here, but if you want to remove these entities from the database uncomment the code below.
        //SaveResult sr = _context.SaveChanges();

      }
      PromptToContinue();
    }

    private void PromptToContinue() {
      Console.WriteLine();
      Console.WriteLine("Press ENTER to continue...");
      Console.ReadLine();
    }

    #region Private Fields
    DomainModelEntityManager _context;
    #endregion Private Fields
  }
}
